#include <stdio.h>
#include <math.h>
#include <complex.h>
#include <string.h>
#include <stdarg.h>
#include <stdlib.h>

#define GNUPLOT_PATH "/etc/alternatives/gnuplot"

double p = #p#;//parámetro del problema (ancho del haz multiplicado por k0)
double ∞ = #inf#;//infinito (para kx adimensional) [disminuir si se aumenta p] nota:es curioso que para p grande, infinito puede ser menor, pero las componentes evanescentes empiezan simpre a partir de 1...
double h = #h#;//tamaño del paso de integración [disminuir si se aumenta p]
double n_par2 = #n_par2#;//epsilon_paralela por mu_paralela
double theta_i = #theta_i#;//0.785;//ángulo de incidencia
double complex mu_par = #mu_par#, mu_ort = #mu_ort#; //componentes del tensor magnético divididas entre μ_0

double a;//factor de anisotropía



//los siguientes parámetros determinan el tamaño de la cuadrícula sobre la que se graficará

double x_min = #x_min#,  x_max = #x_max#,  z_min = #z_min#, z_max = #z_max#;//[aumentar si aumenta p]
double dx = #dx#;//distancia entre puntos de la cuadrícula en dirección x  [aumentar si aumenta p]
double D = #D#; // distancia entre vectores del campo
double dz;//distancia entre puntos de la cuadrícula en dirección y
double * x, * z; //matrices que guardan las coordenadas de los puntos de la malla
int n_x, n_z; //número de puntos de la malla en cada dirección


double ci,  si;//coseno y seno del ángulo de incidencia del haz (se establece en main())
double complex P; //guarda el valor de I*p;
double F;

typedef enum {COORD_NINGUNA, COORD_FILAS, COORD_CRUZ} opc_coordenadas;
typedef enum {GRAF_INTENSIDAD=512, GRAF_ExH=1, GRAF_ExB=2, GRAF_ExM=4, GRAF_MAXIMOS=8, GRAF_RAYO=16, GRAF_ANGULO_K=32, GRAF_ANGULO_S=64, GRAF_DIVExH=128, GRAF_DIVExB=256, GRAF_INTERFAZ = 1024, GRAF_IExH = 2048, GRAF_IExB = 4096, GRAF_E2_IExH = 8192, GRAF_IExB_IExH = 16384, GRAF_FASE = 32768} opc_grafica;

const int grabar_matriz=1;//establecer a un valor distinto de 1 para evitar que se calcule y guarde la matriz
const int imprimir_comandos_gp=0;//determina si se escriben en pantalla los comandos de gnuplot
int n_prom = 0;//número de veces que se promedia la matriz de intensidad
int escalar_ce = #escalar_ce#;//si es 'S', se normalizarán los campos escalares de manera que su máximo valor sea 1
int escalar_cv = #escalar_cv#;//si es 'S', se normalizarán los campos vectoriales de acuerdo a la separación especificada entre puntos de éstos
opc_grafica graf = #graf#;//GRAF_INTENSIDAD + GRAF_MAXIMOS + GRAF_ExB + GRAF_ExH + GRAF_DIVExH + GRAF_DIVExB + GRAF_ANGULO_K;

FILE *gp;
FILE *Gp;

double complex kz_1(double complex);

double complex kz_2(double complex);

double complex fs(double complex);

double complex tm(double complex);

double complex rs(double complex);

double complex k_par(double);

void em1(double, double, double complex*, double complex*, double complex*, double complex*, double complex*);

void em2(double, double, double complex*, double complex*, double complex*, double complex*, double complex*);

double complex Hx(double complex, double);

double complex Hz(double complex, double);

void cimp(complex, char[40]);

double theta_t();

void valores_iniciales();

void g(char*, ...	);

int graficar();

/*void guardar_matriz(int, int, double*, char*, opc_coordenadas, ...);*/

/*void promediar_matriz(int, int, double*);*/

/*void perfil_haz(double, double, double, double, int, int, double*, char*);*/

/*void obtener_maximos(int, int, double*);*/

/*void obtener_rayo(double, double, int, int, double*);*/

int main();

double signo(double x){
	return x == 0 ? 0 : x/cabs(x);
}

double complex kz_1(double complex kx){//devuelve k_z en el lado 1, dado el valor de k_x. Para k_x^2 mayor que 1 se devuelve un número complejo
	return csqrt(1 - cpow(kx, 2));
}

double complex kz_2(double complex kx){//análogo a la función anterior para el lado 2
	return signo(mu_par)*csqrt(n_par2 - a*pow(kx,2));
}

double complex fs(double complex kx){//función auxiliar para los coeficientes de transmisión. Recibe la componente del vector de onda paralela a la interfaz.
	return kz_2(kx)/(kz_1(kx)*mu_par);
}

double complex tm(double complex kx){//amplitud de transmisión. Depende de la componente paralela de k
	return 2/(1+fs(kx));
}

double complex rs(double complex kx){//amplitud de reflexión. Depende de la componente paralela de k
	double complex f = fs(kx);//guardar el valor de f para evitar un doble llamado de la función en el siguiente renglón
	return (1-f)/(1+f);
}

double complex k_par(double kxi){//devuelve la componente del vector de onda paralela a la interfaz a partir de la componente k_x en los ejes del sistema de incidencia
	double complex kzi = kz_1(kxi);
	return kxi*ci + kzi*si;
}

void em1(double x, double z, double complex *α, double complex *αx, double complex *αz, double complex *αxx, double complex *αzz){
	double kxi;//componente del vector de onda ortogonal al eje de incidencia
	double complex kx, kz, X, fi, fr, Z;//componente del vector de onda paralela a la interfaz; componente del vector de onda ortogonal a la inerfaz

	*α = 0;
	*αx = 0;
	*αz = 0;
	*αxx = 0;
	*αzz = 0;
	
	/*
	En las siguientes líneas se hace una integración numérica del campo eléctrico utilizando el método de rectángulos y una partición regular del intervalo (-inf, inf), de tamaño h
	*La elección del valor apropiado de infinito depende de la función que se integre. En este caso, en que se utiliza una función gaussiana, para valores grandes de *p* (el parámetro que mide adimensionalmente el ancho del haz) la función decae muy rápidamente y no es necesario poner infinitos muy "grandes". Para valores menores de *p*  hay que reajustar el valor de infinito.
	*También hay que considerar que, para valores "altos" de *p*, se requiere un menor ancho de los rectángulos para no perder mucha precisión.
	*/
	
	for(kxi = -∞; kxi <= ∞; kxi += h){
		kx = k_par(kxi);//rotar para obtener la componente paralela a la interfaz
		X = -pow(p*kxi, 2)/2 + P*x*kx;//utilizada para evitar un doble cálculo de el cuadrado de kxi
		kz = kz_1(kx);//obtener kz a partir de la relación de dispersión
		Z = P*kz*z;
		
		fi = cexp(X+Z);
		fr = rs(kx)*cexp(X-Z);
		
		*α	+= fi + fr;//campo incidente
			//campo reflejado. Difiere en la componente z de k y en la amplitud
			//estrictamente aquí habría que multiplicar por el tamaño del paso para obtener el área, pero dado que la partición es uniforme, se puede sacar h de la suma y multiplicar al final, evitando una multiplicación en cada paso del ciclo
		*αz += kz*(fi - fr);
		*αx += kx*(fi + fr);
		//!!revisar si aquí debe haber un factor 'P'
		*αxx += kx*kx*(fi + fr);
		*αzz += kz*kz*(fi + fr);
	}
	
	*α *= F;//multiplicar por los factores adecuados y el tamaño del ancho
	*αx *= I*F;
	*αz *= I*F;
	*αxx *= -F;
	*αzz *= -F;
}

void em2(double x, double z, double complex *α, double complex *αx, double complex *αz, double complex *αxx, double complex *αzz){
	double kxi;//componente del vector de onda ortogonal al eje de incidencia
	double complex kx, kz, A, ft;
	
	*α = 0;
	*αx = 0;


	*αz = 0;
	*αxx = 0;
	*αzz = 0;
	
	/*
	En las siguientes líneas se hace una integración numérica del campo eléctrico utilizando el método de rectángulos y una partición regular del intervalo (-inf, inf), de tamaño h
	*La elección del valor apropiado de infinito depende de la función que se integre. En este caso, en que se utiliza una función gaussiana, para valores grandes de *p* (el parámetro que mide adimensionalmente el ancho del haz) la función decae muy rápidamente y no es necesario poner infinitos muy "grandes". Para valores menores de *p*  hay que reajustar el valor de infinito.
	*También hay que considerar que, para valores "altos" de *p*, se requiere un menor ancho de los rectángulos para no perder mucha precisión.
	*/
	
	for(kxi = -∞; kxi <= ∞; kxi += h){
		kx = k_par(kxi);//rotar para obtener la componente paralela a la interfaz
		kz = kz_2(kx);//obtener kz a partir de la relación de dispersión
		A = cexp(-pow(p*kxi, 2)/2 + P*x*kx + P*z*kz);//utilizada para evitar un doble cálculo de el cuadrado de kxi
		
		ft = tm(kx)*A;
		
		*α	+= ft;//campo transmitido
			//campo reflejado. Difiere en la componente z de k y en la amplitud
			//estrictamente aquí habría que multiplicar por el tamaño del paso para obtener el área, pero dado que la partición es uniforme, se puede sacar h de la suma y multiplicar al final, evitando una multiplicación en cada paso del ciclo
		*αz += kz*ft;
		*αx += kx*ft;
		
		*αxx += kx*kx*ft;
		*αzz += kz*kz*ft;
	}
	
	*α *= F;//multiplicar por los factores adecuados y el ancho
	*αx *= I*F;
	*αz *= I*F;
	*αxx *= -F;
	*αzz *= -F;
}


double complex μpar(double z){
	return z < 0 ? 1 : mu_par;
}

double complex μort(double z){
	return z < 0 ? 1 : mu_ort;
}

double complex Hx(double complex Bx, double z){
	return Bx/(z < 0 ? 1 : mu_par);
}

double complex Hz(double complex Bz, double z){
	return Bz/(z < 0 ? 1 : mu_ort);
}


void cimp(complex var, char nombre[40]){//esta función se utiliza para imprimir valores de algunas variables (sólo para hacer pruebas del programa)
	printf(nombre);
	printf(": %e\t+\t%e \t i\n", creal(var), cimag(var));
}

double theta_t2(){//esta función calcula el ángulo de transmisión con base en criterios de intensidad [en desarrollo]
	double complex Sz = 0;//variable para sumar (contiene el valor de la integral en cada paso)
	double complex Sx = 0;
	double kxi;//componente del vector de onda ortogonal a los ejes propios (adimensionalizado)
	double complex kx;
	double complex kz;
	double A;
	double complex E = 0;//E2(0,1);
	double t;
	double h = 3;

	for(kxi = -∞; kxi <= ∞; kxi += h){
		A = -pow(p*kxi, 2)/2;
		kx = k_par(kxi);
		kz = kz_2(kx);
		Sx += tm(kx)*cexp(A)*I*kx;
		Sz += tm(kx)*cexp(A)*I*kz;
	}
	

	//printf("%e %e\n", ( cabs(E2(0,h) - cabs(E2(0,0))) )/h ,  ( cabs(E2(h,0)) - cabs(E2(0,0)) )/h );
	//printf("%e\n", -( cabs(E2(0,h) - cabs(E2(0,0))) ) / ( cabs(E2(h,0)) - cabs(E2(0,0)) ) );
	
	//t = -atan( ( cabs(E2(0,1) - cabs(E2(0,0))) ) / ( cabs(E2(1,0)) - cabs(E2(0,0)) ) );
	printf("%e\n", t );
	return t;
}

double Theta_t(){
	return signo(mu_par)*asin(si/sqrt(n_par2+(1-a)*pow(si,2)));
}

double theta_t(){
	return asin(abs(mu_par)*si/(mu_ort*sqrt(n_par2 + a*(a-1)*pow(si,2))));
}

void valores_iniciales(){
	int i;
	a = mu_par/mu_ort;
	
	//establecer igual desplazamiento en ambas direcciones
	dz = dx;
	
	//guardar senos y cosenos del ángulo de incidencia (evita hacer múltiples llamadas a estas funciones en los muchos ciclos del programa)
	ci = cos(theta_i);
	si = sin(theta_i);
	
	P = I*p;
	F = p*h;
	
	n_z = 1 + (int) ((z_max - z_min) / dz);
	n_x = 1 + (int) ((x_max - x_min) / dx);
	
	//nota: hay que cuidar que n_z * n_x no exceda 1047342;
	
	
	x = (double*) calloc (n_x, sizeof(double));
	z = (double*) calloc (n_z, sizeof(double));
	
	for(i = 0; i < n_x; i++){
		x[i] = x_min + i*dx;
	}
	
	for(i = 0; i < n_z; i++){
		z[i] = z_min + i*dz;
	}
}

void escalar_campo(int n, int m, double (*M)[n][m], int c, int C, double nuevo_max){
	int i;
	double norma;
	double max = 0;
	
	for(i = 0; i < m; i++){
		norma = sqrt(pow((*M)[c][i], 2) + pow((*M)[C][i], 2));
		if(max<norma) max = norma;
	}
	
	max /= nuevo_max;
	
	if(max >0 )
		for(i = 0; i < m; i++){
			(*M)[c][i] = (*M)[c][i] / max;
			(*M)[C][i] = (*M)[C][i] / max;
		};
}


void escalar_matriz(int n, int m, double (*M)[n][m], double nuevo_max){
	int i, j;
	double max = 0;
	
	for(i = 1; i < n; i++)
		for(j = 1; j < m; j++)
			if((*M)[i][j] > max) max = (*M)[i][j];
	
	max /= nuevo_max;
	
	for(i = 1; i < n; i++)
		for(j = 1; j < m; j++)
			(*M)[i][j] /= max;
}

void guardar_matriz(int n, int m, double M[n][m], char *nombre_archivo, opc_coordenadas agregar_coordenadas, ...){
	int i, j, salto = 1, k;
	FILE *archivo;
	
	archivo = fopen(nombre_archivo, "w");
	
	
	va_list args;
	va_start(args, agregar_coordenadas);
	if(agregar_coordenadas)	salto = va_arg(args, int);
	va_end(args);
	
	int u = 1 + (int) (n_z/salto);

	switch(agregar_coordenadas){
		case COORD_CRUZ:
			M[0][0] = n - 1;
			for(i = 1; i < n; i++) M[i][0] = z[i * salto-1];
			for(j = 1; j < m; j++) M[0][j] = x[j * salto-1];
			break;
		case COORD_FILAS:
			for(k = 0; k < m; k++){
				M[0][k] = z[k % u * salto]; //coordenada z
				M[1][k] = x[k / u * salto]; //coordenada x
			}
		break;
	}
	
	
	for(j = 0; j < m; j++){
		
		for(i = 0; i < n; i++){
			fprintf(archivo, "%f\t", M[i][j]);
		}
		fprintf(archivo, "\n");
	}
	
	fclose(archivo);
}

void promediar_matriz2(int n, int m, double (*A)[n][m]){
	int d = 3, dn,dm;
	double B[n][m], S;
	int J, K, j, k, N;
	
	for(J = 0; J < n; J++){
		dn = d;
		if(dn < J && J < d) dn = J;
		if(J + dn >= n) dn = n - J - 1;
		for(K=0; K < m; K++){
			dm = dn;
			if(dm < K && K < dn) dm = K;
			if(K + dm >= m) dm = m - K - 1;
			S = 0;
			N = 0;
/*			printf("%d\t", dm);*/
			for(j = J - dm; j <= J + dm; j++){
				for(k = K - dm; k <= K + dm; k++){
					S += (*A)[j][k];
					N++;
				}
			}
			B[J][K] = S/N;
		}
	}
	
	
	for(J = 0; J < n; J++){
		for(K=0; K < m; K++) (*A)[J][K] = B[J][K];
	}
}

void promediar_matriz(int n, int m, double (*A)[n][m]){
	int d = 1;
	double B[n][m], S;
	int J, K, j, k, N;
	
	for(J = 0; J < n; J++){
		for(K=0; K < m; K++){
			S = 0;
			N = 0;
/*			printf("%d\t", dm);*/
			for(j = J - d; j <= J + d; j++){
				for(k = K - d; k <= K + d; k++){
					if(j>0 && j < n && k > 0 && k <m){
						S += (*A)[j][k];
						N++;
					}
				}
			}
			B[J][K] = S/N;
		}
	}
	
	
	for(J = 0; J < n; J++){
		for(K=0; K < m; K++) (*A)[J][K] = B[J][K];
	}
}


int opc(opc_grafica opcion){
	return (graf & opcion) == opcion;
}

void perfil_haz(double z0, double x0, double pendiente, double longitud, int N, int M, double E[N][M],char *nombre_archivo){
	int n0 = (int) ((z0 - z_min)/dz), m0 = (int) ((x0 - x_min)/dx);
	int n, m;
	int n_max = (int) (longitud/(2*sqrt(1 + pow(pendiente,2))*dz));
	FILE *archivo;
	if(n_max > n0) n_max = n0;
	if(n0 + n_max > N) n_max = N - n0;
	archivo = fopen(nombre_archivo, "a");
	
/*	printf("%f\t%f\t%f\t%d\t%d\t%d\n", z0, x0, pendiente, n0, m0, n_max);*/
	
	for(n = n0 - n_max; n < n0 + n_max; n++){
		m = (int) (m0 + pendiente*(n-n0));
/*		printf("%d, %d \t ", n, m);*/
		fprintf(archivo, "%f\t%f\n", (n > n0 ? 1.0 : -1.0)*sqrt( pow(dz*(n-n0), 2) + pow(dx*(m-m0), 2) ), E[n][m]);
	}
	
	fprintf(archivo, "\n\n");
	fclose(archivo);
}

void obtener_maximos(int N, int M, double E[N][M]){
	int n, m;
	int Dz, Dx;
	double coord[2][N-1];
	double actual;
	double max;

	
	FILE *perfil;
	
	perfil = fopen("#prefijo#-perfil.dat", "w");
	
	fclose(perfil);
	
	for(n = 0; n < N-1; n++){
		coord[0][n] = z[n];
		max = 0;
		
		for(m = 0; m < M-1; m++){
			actual = E[n+1][m+1];
			if(max <= actual){
				max = actual;
				coord[1][n] = x[m];
			}
		}
		
		if(n > 1) perfil_haz(coord[0][n], coord[0][n], dz/(coord[1][n] - coord[1][n-1]), 20, N, M, E, "#prefijo#-perfil.dat");
		
	}
	
	guardar_matriz(2, n, coord, "#prefijo#-maximos.dat", COORD_NINGUNA);
}


void obtener_rayo(double z0, double x0, int N, int M, double E[N][M]){
	double r = 1.0/sqrt(2);
	int n = (int) (z0-z_min)*n_z/(z_max-z_min), m = (int) (x0-x_min)*n_x/(x_max-x_min), n_min, m_min, i, i_min;
	int k = 4;
	double min;
	double dif[k];
	int dm[] = {1, 1, -1, 0}, dm_ant = 2;
	int dn[] = {0, 1, 1, 1}, dn_ant = 2;
	double h[] = {1, r, 1, r}, e;
	int L=0;
	
	FILE *archivo = fopen("#prefijo#-rayo.dat", "w");
	
	//printf("z0:%f\tx0:%f\n", z0, x0);
	
	while(n<N-1 && m<M-1 && n>=0 && m>=0 && L < 500){
		fprintf(archivo, "%f\t%f\n", z[n], x[m]);
		
/*		printf("n:%d\tm:%d\tE:%f\n", n, m, E[n][m]);*/
		e = E[n][m];
		for(i=0; i<k; i++){
			dif[i] = h[i]*fabs(e - E[n+dn[i]][m+dm[i]]);
/*			printf("dn:%d\tdm:%d\td:%f\tE:%f\n", dn[i], dm[i], dif[i], E[n+dn[i]][m+dm[i]]);*/
		}
		
/*		printf("\n");*/
		
		min = 1000000;
		
		for(i=0; i<k; i++){
			if(min > dif[i] && !( dn_ant==-dn[i] && dm_ant==-dm[i]) ){
				i_min = i;
				min =  dif[i];
				n_min = n + dn[i];
				m_min = m + dm[i];
			}
		}
		dn_ant = dn[i_min];
		dm_ant = dm[i_min];
		m = m_min;
		n = n_min;
/*		printf("%d\t%d\n", n, m);*/
		L++;
	}
	fclose(archivo);
}


int main(int n_args, char *args[]){
	double complex α, αz, αx, αxx, αzz, αc;
	double u, i1, i2;
	int progreso_actual = -1;
	double progreso = -1;
	
	int i_z, i_x = 0, i = 0;	

	int salto = (int) (D/dx);

	valores_iniciales();
	
	int N_x = 1 + (int) ( (double) n_x / salto );
	int N_z = 1 + (int) ( (double) n_z / salto );
	int N = N_z*N_x;
	int M_z = 1 + n_z;
	int M_x = 1 + n_x;

	double E[M_z][M_x], F[M_z][M_x], GF[M_z][M_x], divExH[M_z][M_x], divExB[M_z][M_x], I1[M_z][M_x], I2[M_z][M_x], P1[M_z][M_x], P2[M_z][M_x];
	double ExH[4][N], ExB[4][N], ExM[4][N], CN[4][N];
	double params[3][1];

	if(grabar_matriz==1){//esta variable se puede establecer manualmente a un valor distinto de 1 para evitar el procesamiento de la matriz
	
		printf("Calculando...    ");
		//fflush(stdout);
		
		setbuf(stdout, NULL);

		for(i_x = 1; i_x <= n_x; i_x++){//recorrer todos los posibles valores de x en el rectángulo
			
			
			for( i_z = 1; i_z <= n_z; i_z++ ){
				z[i_z-1] < 0 ? em1(x[i_x-1], z[i_z-1], &α, &αx, &αz, &αxx, &αzz) : em2(x[i_x-1], z[i_z-1], &α, &αx, &αz, &αxx, &αzz);
				αc = conj(α);
				u = αc*α/2;	//densidad de energía
				E[i_z][i_x] = u;
				F[i_z][i_x] = cacos(creal(α)/u);	//fase
				i1 = sqrt(pow(cimag(αc*αz/μpar(z[i_z-1])),2) + pow(cimag(αc*αx/μort(z[i_z-1])),2));	//norma de ExH
				i2 = sqrt(pow(cimag(αc*αz),2)+pow(cimag(αc*αx),2));	//norma de ExB
				I1[i_z][i_x] = i1;
				I2[i_z][i_x] = i2;
				P1[i_z][i_x] = u/i1;	//proporción entre E^2 y ||ExH||
				P2[i_z][i_x] = i2/i1;	//proporción entre ||ExB|| y ||ExH||
				GF[i_z][i_x] = sqrt(pow(cimag(αc*αz),2)+pow(cimag(αc*αx),2))/(αc*α);//norma del gradiente de la fase 	
				//if(abs(GF[i_z][i_x]) > 2 ) GF[i_z][i_x] = 2;
				
				divExB[i_z][i_x] = cimag(αc*(αxx + αzz));
				divExH[i_z][i_x] = cimag(αc*αxx/μort(z[i_z-1]) + αc*αzz/μpar(z[i_z-1]));
				if( i_z % salto == 1 && i_x % salto == 1 ){
					ExB[2][i] = cimag(αc*αz);
					ExB[3][i] = cimag(αc*αx);
					
					ExH[2][i] = cimag(αc*αz/μpar(z[i_z-1]));
					ExH[3][i] = cimag(αc*αx/μort(z[i_z-1]));
					
					D = 1/sqrt(1+pow(cimag(αx)/cimag(αz), 2));
					CN[2][i] = D;
					CN[3][i] = D*cimag(αx)/cimag(αz);
					
					ExM[2][i] = ExB[2][i] - ExH[2][i];
					ExM[3][i] = ExB[3][i] - ExH[3][i++];
					
				}
				
			}
			
			progreso = ((double) i_x)/n_x*100;
			
			if( progreso >= progreso_actual +1){
				progreso_actual = (int) progreso;
				printf("\b\b\b%2d%%", progreso_actual);
				fflush(stdout);
			}
		}
		
		puts("");
		
		if(n_prom > 0) puts("Promediando...");
		for(i = 0; i < n_prom; i++) promediar_matriz(M_z, M_x, &E);
		for(i = 0; i < n_prom; i++) promediar_matriz(M_z, M_x, &F);
		
		if(escalar_cv){
			puts("Escalando campos vectoriales...");
			escalar_campo(4, N, &ExB, 2, 3, D);
			escalar_campo(4, N, &ExH, 2, 3, D);
			escalar_campo(4, N, &ExM, 2, 3, D);
		}
		
		if(escalar_ce){
			puts("Escalando campos escalares...");
			escalar_matriz(M_z, M_x, &E, 1);
		}
		
		puts("Guardando datos...");
		
		if(opc(GRAF_INTENSIDAD)) 
			guardar_matriz(M_z, M_x, E, "#prefijo#-intensidad.dat", COORD_CRUZ, 1);
		if(opc(GRAF_IExH)) 
			guardar_matriz(M_z, M_x, I1, "#prefijo#-IExH.dat", COORD_CRUZ, 1);
		if(opc(GRAF_IExB)) 
			guardar_matriz(M_z, M_x, I2, "#prefijo#-IExB.dat", COORD_CRUZ, 1);
		if(opc(GRAF_E2_IExH)) 
			guardar_matriz(M_z, M_x, P1, "#prefijo#-E2_IExH.dat", COORD_CRUZ, 1);
		if(opc(GRAF_IExB_IExH)) 
			guardar_matriz(M_z, M_x, P2, "#prefijo#-IExB_IExH.dat", COORD_CRUZ, 1);
		if(opc(GRAF_FASE)) 
			guardar_matriz(M_z, M_x, F, "#prefijo#-fase.dat", COORD_CRUZ, 1);
		/*if(opc(GRAF_OTROS)) 
			guardar_matriz(M_z, M_x, GF, "#prefijo#-gradiente-fase.dat", COORD_CRUZ, 1);*/
		if(opc(GRAF_DIVExH)) 
			guardar_matriz(M_z, M_x, divExH, "#prefijo#-divExH.dat", COORD_CRUZ, 1);
		if(opc(GRAF_DIVExB)) 
			guardar_matriz(M_z, M_x, divExB, "#prefijo#-divExB.dat", COORD_CRUZ, 1);
		if(opc(GRAF_ExB)) 
			guardar_matriz(4, N, ExB, "#prefijo#-ExB.dat", COORD_FILAS, salto);
		if(opc(GRAF_ExH)) 
			guardar_matriz(4, N, ExH, "#prefijo#-ExH.dat", COORD_FILAS, salto);
		if(opc(GRAF_ExM)) 
			guardar_matriz(4, N, ExM, "#prefijo#-ExM.dat", COORD_FILAS, salto);
		/*if(opc(GRAF_OTROS)) 
			guardar_matriz(4, N, CN, "#prefijo#-CN.dat", COORD_FILAS, salto);*/
	}
	

	obtener_rayo(7*dz, 3*dx, M_z, M_x, E);
	obtener_maximos(M_z, M_x, E);
	params[0][0] = theta_i;
	params[1][0] = Theta_t();
	params[2][0] = theta_t();
	
	guardar_matriz(3, 1, params, "#prefijo#-params.dat", COORD_NINGUNA); 
	return 0;
}
